#!/usr/bin/perl -w

use CGI qw/:standard *blockquote *table *Tr *div/;
# http://www.geocities.co.jp/SiliconValley-Sunnyvale/6128/perl/cgipm.html
# http://www.tohoho-web.com/wwwperl1.htm

sub ln { "\n"; }

sub image {
    my ($e) = @_;
    "<img src=\"$e.png\" alt=\"$e\" align=\"middle\">";
}

$NATURAL = '<img src="natural.png">';  # without align

my $dim_min     = 1;
my $dim_default = 3;
my $dim_max     = 10;
my $dim = param('dim');
unless (defined($dim) && $dim_min <= $dim && $dim <= $dim_max) {
    $dim = $dim_default;
    param('dim', $dim);
}

sub change_dim {
    $dim = int(param('newdim'));
    unless ($dim_min <= $dim && $dim <= $dim_max) {
	$dim = $dim_default;
    }
    param('dim', $dim);
}

sub print_header {
print
    header(-expires=>'+1m'), # cache expires in 1min
    start_html(-title=> 'Pseudo-Separable Absolute L#-Convex'),
    a({href=>"./index.en.html"},'DCP HOME'), ln,
    h1('Pseudo-Separable Absolute L<img src="natural.png" width=14>-Convex Minimization'), ln,
    start_blockquote, ln;
}

sub set_param {
    my ($index, $min, $max) = @_;
    my ($e) = (param($index));
    if ( defined($e) && $e =~ m/([-+]?\d+)/
	 && $min <= int($1) && int($1) <= $max ) {
	$e = int($1);
    } else {
	$e = $min + int(rand($max-$min+1));
    }
    param($index, $e);
}

sub init_param {
    my ($i, $j);

    for ($i=0; $i<$dim; $i++) {
	for ($j=0; $j<$i; $j++) {
	    set_param("s:$i:$j", 0, 50);
	    set_param("t:$i:$j", -50, 50);
	}
	set_param("a:$i", 1, 50);
	set_param("b:$i", -50, 50);
	set_param("c:$i", -50, 50);
	set_param("x:$i", -50, 50);
    }
}

sub print_html_table {
    print 'For a finite set V={1,2,...,n} and a family of univariate discrete convex functions ',
    image('f_ij'), ' and ', image('f_i'),' indexed by ', image('ijV'), ' and ',image('iV'),
    ', the function defined by ', ln;
    print p,image('def-lseparable'), '  ',
    image('def-pluslseparable'), ln;
    print p, 'is an L','<img src="natural.png">',
    '-convex function [1,2]. ';
#  and a separable convex function is L', '<img src="natural.png">', '-convex [1,2].';
#  print 'The sum of an L-convex function and an L', '<img src="natural.png">',
#  '-convex function is an L', '<img src="natural.png">', '-convex function [1,2].';
    print p, 'Here we consider a absolute L',
    '<img src="natural.png">', '-convex function:',
    ln;

    print p, '<img src="objfunc-separable-absLpiri.png">';
    print p, 'In this web application, you can choose the dimension n from 1 to 10.';

    print start_form, hidden('dim'), start_div({-align=>'center'}), ln;
    @index = ($dim_min..$dim_max);
    print p, 'dimension n = ', popup_menu(-name=>'newdim',-values=>[@index],-default=>$dim);
    print submit('Change'), ln;

    print p, 'You can input parameters s', Sub(em('ij')), ', t', Sub(em('ij')),
    ', a', Sub(em('i')),' and b', Sub(em('i')),
#    ' and c', Sub(em('i')),
  ' in the ranges ';
    print '0 < s', Sub(em('ij')) ,', a', Sub(em('i')),
    '<100 and -100< t',
    Sub(em('ij')),
    ', b', Sub(em('i')),
  #', c', Sub(em('i')),
    '< 100. ';
    print p, 'You can also input an initial solution x.';
  
    foreach $e ('s', 't') {
	print br, start_table({-border=>'1'}), ln;
	print Tr({-align=>'right'}, [td({-width=>int(100/(1+$dim)).'%'},
					[ $e, 1 .. $dim ])]);
	for ($i=0; $i<$dim; $i++) {
	    my @td = ($i+1);
	    for ($j=0; $j<$i; $j++) {
		push @td, textfield(-name=>"$e:$i:$j", -size=>3, -default=>param("$e:$i:$j"));
	    }
	    print Tr({-align=>'right'}, [td([@td])]), ln;
	}
	print end_table, ln;
    }

    print br, start_table({-border=>'1'}), ln;
    print Tr({-align=>'right'}, [td({-width=>int(100/(1+$dim)).'%'},
				    [ '', 1 .. $dim ])]);
    foreach $e ('a', 'b', 'x') {
	my @td = ($e);
	for ($j=0; $j<$dim; $j++) {
	    push @td, textfield(-name=>"$e:$j", -size=>3, -default=>param("$e:$j"));
	}
	print Tr({-align=>'right'}, [td([@td])]), ln;
    }
    print end_table, ln;

    print p, '(Note: "Reset" button resets your inputs to default values.', ln;
    print br, 'An inappropriate input value will be replaced with a random number.)', ln;
    print p, submit('Minimize'), reset('Reset'), ln;
    print end_div, end_form, ln;
}

sub print_reference {
    print <<EOF;
<hr width="70%">
<p> This web application minimizes f(x) using
<a href="http://ist.ksc.kwansei.ac.jp/~tutimura/odicon/index.en.html">ODICON</a>. 

<p> [1] K. Murota (2001): "Discrete Convex Analysis---An Introduction (in Japanese),"
Kyoritsu Publishing Company, Tokyo. Section 5.2.

<p> [2] K. Murota (2003):
"Discrete Convex Analysis," SIAM. Section 7.3.

<hr width="70%">
EOF
# <p> [3] K. Murota,
# Recent developments in discrete convex analysis,
# in: W. Cook, L. Lovasz and J. Vygen, eds.,
# Research Trends in Combinatorial Optimization, Bonn 2008,
# Springer-Verlag, Berlin, (2009), Chapter 11, 219--260.
}

sub call_executable {
    print '<hr width="70%">', ln;
    print '<pre>', ln;
    open(OUT, "|./odicon DLA $dim") || die('Can not call ./odicon');
    #open(OUT, '|cat') || die('Can not call ./odicon');
    foreach $e ('s', 't') {
	for ($i=0; $i<$dim; $i++) {
	    for ($j=0; $j<$i; $j++) {
		print OUT param("$e:$i:$j"), ' ';
	    }
	    print OUT ln;
	}
    }
    foreach $e ('a', 'b', 'c', 'x') {
	for ($j=0; $j<$dim; $j++) {
	    print OUT param("$e:$j"), ' ';
	}
	print OUT ln;
    }
    close OUT;
    print '</pre>', ln;
}


change_dim() if param('Change');
init_param();
print_header();
print_html_table();
call_executable() if param('Minimize');
print_reference();

print
    '<div align="right"><address>Nobuyuki Tsuchimura, Satoko Moriguchi</address></div>',ln,
    end_blockquote, end_html, ln;
