#!/usr/bin/perl -w

use CGI qw/:standard *blockquote *table *Tr *td *div/;
# http://www.geocities.co.jp/SiliconValley-Sunnyvale/6128/perl/cgipm.html
# http://www.tohoho-web.com/wwwperl1.htm

sub ln { "\n"; }

sub image {
    my ($e) = @_;
    "<img src=\"$e.png\" alt=\"$e\" align=\"middle\">";
}

$NATURAL = '<img src="natural.png">';  # without align

my $dim_min     = 1;
my $dim_default = 3;
my $dim_max     = 9;
my $dim = param('dim');
unless (defined($dim) && $dim_min <= $dim && $dim <= $dim_max) {
    $dim = $dim_default;
    param('dim', $dim);
}

sub change_dim {
    $dim = int(param('newdim'));
    unless ($dim_min <= $dim && $dim <= $dim_max) {
	$dim = $dim_default;
    }
    param('dim', $dim);
}

sub print_header {
print
    header(-expires=>'+1m'), # cache expires in 1min
    start_html(-title=> 'Laminar Logarithmic M#-Convex'),
    a({href=>"./index.en.html"},'DCP HOME'), ln,
    h1('Laminar Logarithmic M<img src="natural.png" width=14>-Convex Minimization'), ln,
    start_blockquote, ln;
}

sub set_param {
    my ($index, $min, $max) = @_;
    my ($e) = (param($index));
    if ( defined($e) && $e =~ m/([-+]?\d+)/
	 && $min <= int($1) && int($1) <= $max ) {
	$e = int($1);
    } else {
	$e = $min + int(rand($max-$min+1));
    }
    param($index, $e);
}

sub init_param {
    my ($i);

    for ($i=0; $i<$dim*2-1; $i++) {
	set_param("a:$i",   1, 9);
	set_param("b:$i",   1, 9);
    }
    for ($i=0; $i<$dim; $i++) {
	set_param("x:$i", 1, 99);
    }
}

sub print_html_table {
  print 'A laminar family means a nonempty family ',
      image('calT'), ' of subsets of {1, ... , n} such that', ln;
  print image('def-laminar');
  print p, 'For a laminar family ',
      image('calT'), ' and a family of univariate discrete convex functions ',
      image('f_Y'), ' indexed by ',
      image('YinT'), ', the function defined by ', ln;
  print image('def-laminar-conv');
  print p, "is an M$NATURAL-convex function [1,2], where ",
  image('def-x-Y'), ln;
  print p, 'Here we consider a logarithmic laminar M',
  '<img src="natural.png">', '-convex function:',
  ln;
# f(x)=\sum_{Y\in \cal {T}}\{-a_Y\log (b_Yx(Y))\}
print p,image('objfunc-separable-logpxM');
    print p, 'for a laminar', image('calT'), 
      '. In this web application, the laminar family is preset for each dimension.
For example, ';



    print        
	image('calT'), 
	' = {{1},{2},{3}, {1,2}, {1,2,3}} in the 3-dimensional case.', ln;

    print p, 'You can choose the dimension n from 1 to 9.';

    print start_form, hidden('dim'), start_div({-align=>'center'}), ln;
    @index = ($dim_min..$dim_max);
    print p, 'dimension n = ', popup_menu(-name=>'newdim',-values=>[@index],-default=>$dim);
    print submit('Change'), ln;

print p, 'You can also input parameters a', Sub(em('Y')), ' and b', Sub(em('Y')), ' in the ranges ';
    print '0 < a', Sub(em('Y')) , ', b',
    Sub(em('Y')), '< 10. ';

    print start_table({-border=>'1'}), ln;
    my @td = 1..$dim;
    foreach $i (@td) { $i =~ s/(.*)/\{$1\}/; }
    print Tr({-align=>'center'}, td([@td])), ln;
    my $total = 0;
    for ($i=1; $i/2<$dim; $i*=2) {
	print start_Tr({-align=>'center'});
	for ($j=0; $j+$i/2<$dim; $j+=$i) {
	    my $max = $j+$i;  $max = $dim if $max > $dim;
	    my $x = 'x({' . join(',',($j+1)..$max) . '})';
	    print start_td({-colspan=>$i});
	    print '-', textfield(-name=>"a:$total", -size=>3, -default=>param("a:$total")),'log(';
            print textfield(-name=>"b:$total", -size=>3, -default=>param("b:$total")), $x,')+', $x;
            print end_td, ln;
	    $total++;
	}
	print end_Tr, ln;
    }
    print end_table, ln;
print p, 'You can also input an initial solution x:';
    print br, start_table({-border=>'1'}), ln;
    print Tr({-align=>'right'}, [td({-width=>int(100/(1+$dim)).'%'}, [ '', 1 .. $dim ])]);
    foreach $e ('x') {
	my @td = ($e);
	for ($j=0; $j<$dim; $j++) {
	    push @td, textfield(-name=>"$e:$j", -size=>3, -default=>param("$e:$j"));
	}
	print Tr({-align=>'right'}, [td([@td])]), ln;
    }
    print end_table, ln;

    print p, '(Note: "Reset" button resets your inputs to default values.', ln;
    print br, 'An inappropriate input value will be replaced with a random number.)', ln;
    print p, submit('Minimize'), reset('Reset'), ln;
    print end_div, end_form, ln;
}

sub print_reference {
    print <<EOF;
<hr width="70%">
<p> This web application minimizes f(x) using
<a href="http://ist.ksc.kwansei.ac.jp/~tutimura/odicon/index.en.html">ODICON</a>.

<p> [1] K. Murota (2001): "Discrete Convex Analysis---An Introduction (in Japanese),"
Kyoritsu Publishing Company, Tokyo. Section 4.3.

<p> [2] K. Murota (2003):
"Discrete Convex Analysis," SIAM. Section 6.3.

<hr width="70%">
EOF
}

sub call_executable {
    print '<hr width="70%">', ln;
    print '<pre>', ln;
    open(OUT, "|./odicon DML $dim") || die('Can not call ./odicon');
    #open(OUT, '|cat') || die('Can not call ./odicon');
    foreach $e ('a', 'b') {
	for ($j=0; $j<$dim*2-1; $j++) {
	    print OUT param("$e:$j"), ' ';
	}
	print OUT ln;
    }
    for ($j=0; $j<$dim; $j++) {
	print OUT param("x:$j"), ' ';
    }
    print OUT ln;
    close OUT;
    print '</pre>', ln;
}


change_dim() if param('Change');
init_param();
print_header();
print_html_table();
call_executable() if param('Minimize');
print_reference();

print
    '<div align="right"><address>Satoko Moriguchi, Nobuyuki Tsuchimura</address></div>',ln,
    end_blockquote, end_html, ln;
