#!/usr/bin/perl -w
use strict;
use warnings;
use CGI qw/:standard *blockquote *table *Tr *div/;
# http://www.geocities.co.jp/SiliconValley-Sunnyvale/6128/perl/cgipm.html
# http://www.tohoho-web.com/wwwperl1.htm

sub ln { "\n"; }

sub image {
    my ($e, @align) = @_;
    img({-src=>"$e.png", -alt=>"$e", @align});
    #"<img src=\"$e.png\" alt=\"$e\">";
}

my $NATURAL = '<img src="natural.png">';  # without align
my @MOUSE   = (-onMouseOut=>"check()", -onBlur=>"check()");

my $dim = 13;

param('dim', $dim);
param('criterion:min', 1);
param('criterion:max', 20);
param('mu:min', 1);
param('mu:max', 99);
param('rate:min', 1);
param('rate:max', 99);
param('y:min', 1);
param('y:max', 99);

sub print_header {
print
    header(-expires=>'+1m'), # cache expires in 1min
    start_html(-title=> 'Shift Scheduling Problem: L#-Convex Minimization',
	       -head=>[meta( { '-http_equiv' => 'Content-style-type',
			       '-content'    => 'text/html' } ),
		       meta( { '-http_equiv' => 'Content-script-type',
			       '-content'    => 'text/javascript' } )],
	       -onload=>'init()'),
    ln, <<EOF,
<script type="text/javascript">
<!--
var r;
var y;
var out_range;
function init() {
    r = new Array($dim);
    for (i=0; i<$dim; i++) {
	r[i] = document.p2.elements['rate:'+i];
    }
    y = new Array(4);
    for (i=0; i<4; i++) {
	y[i] = document.p2.elements['y:'+i];
    }

    document.p2.reset();
    var mu = document.p2.mu.value - 0;
    min = Math.max(r[0].value, r[1].value) / mu;
    y[0].value = min + 3;
    min = Math.max(r[2].value/mu-y[0].value, r[3].value/mu-y[0].value,
                   r[4].value/mu-y[0].value, 0);
    y[1].value = min + 3;
    min = Math.max(r[5].value/mu-y[1].value, r[6].value/mu-y[1].value,
		   r[7].value/mu, 0);
    y[2].value = min + 3;
    min = Math.max(r[8].value/mu-y[2].value, r[9].value/mu-y[2].value,
		   r[10].value/mu, r[11].value/mu, r[12].value/mu, 0);
    y[3].value = min + 3;
    check();
    return false; // don't continue for reset button
}
function range_check(e, min, max) {
    e.value = ~~e.value - 0;  // parseInt(e.value);
    if (min <= e.value && e.value <= max) {
	e.style.backgroundColor = 'white';
    } else {
	e.style.backgroundColor = 'pink';
	out_range = true;
    }
}
function check() {
    var p2 = document.p2;
    var dummy = 0;
    var list = {'criterion':dummy, 'mu':dummy};
    out_range = false;

    for (var v in list) {
	var min = ~~p2.elements[v+':min'].value - 0;
	var max = ~~p2.elements[v+':max'].value - 0;
	range_check(p2.elements[v], min, max);
    }
    var min = ~~p2.elements['rate:min'].value - 0;
    var max = ~~p2.elements['rate:max'].value - 0;
    for (i=0; i<$dim; i++) {
	range_check(r[i], min, max);
    }
    min = ~~p2.elements['y:min'].value - 0;
    max = ~~p2.elements['y:max'].value - 0;
    var sum = 0;
    for (i=0; i<4; i++) {
	range_check(y[i], min, max);
	sum += y[i].value - 0;
    }
    p2.n_total.value = sum;

    var mu = p2.mu.value - 0;
    min = Math.max(r[0].value, r[1].value) / mu;
    range_check(y[0], min, max);
    min = Math.max(r[2].value/mu-y[0].value, r[3].value/mu-y[0].value,
		   r[4].value/mu-y[0].value);
    range_check(y[1], min, max);
    min = Math.max(r[5].value/mu-y[1].value, r[6].value/mu-y[1].value,
		   r[7].value/mu);
    range_check(y[2], min, max);
    min = Math.max(r[8].value/mu-y[2].value, r[9].value/mu-y[2].value,
		   r[10].value/mu, r[11].value/mu, r[12].value/mu);
    range_check(y[3], min, max);

    if (out_range) {
	p2.Optimize.disabled = true;
    } else {
	p2.Optimize.disabled = false;
    }
    return !out_range;
}

function popup_window(form) {
    if (!check()) return;
    obj = window.open("", "popup", "width=400, height=400, scrollbars=yes, menubar=no, location=no, directories=no");
    form.target = "popup";
    form.action = "./odicon.cgi";
    obj.focus();
    form.submit();
}
// http://www.phppro.jp/qa/609
// -->
</script>
EOF
    a({href=>"./"},'DCP HOME'), ln,
    h1('Shift Scheduling Problem: L<img src="natural.png" width=14>-Convex Minimization'), ln,
    start_blockquote, ln;
}

sub set_param {
    my ($index, $min, $max) = @_;
    my ($e) = (param($index));
    if ( defined($e) && $e =~ m/([-+]?\d+)/
	 && $min <= int($1) && int($1) <= $max ) {
	$e = int($1);
    } else {
	$e = $min + int(rand($max-$min+1));
    }
    param($index, $e);
}

sub init_param {
    my ($i, $j);

    foreach my $e ('criterion', 'mu') {
	my $min = param("$e:min");
	my $max = param("$e:max");
	set_param("$e", $min, $max);
    }

    for ($i=0; $i<$dim; $i++) {
	set_param("rate:$i", param('rate:min'), param('rate:max'));
    }
    for ($i=0; $i<4; $i++) {
	param("y:$i", '');  # set by init() javascript
    }
}

sub print_html_table {
    print <<EOF;
<p>
We consider the shift scheduling problem
in call centers [1].
Suppose
a call center is operational during I=13 intervals.
At the beginning of K=4 specified intervals, employees can start
working, and each employee works for M=5 consecutive intervals.
Shift k starts at the I<sub>k</sub>-th interval
and thus finishes at the beginning of interval I<sub>k</sub> +M. We
assume that
  I<sub>1</sub> = 1, I<sub>2</sub> = 3, I<sub>3</sub> = 6, I<sub>4</sub> = 9.
<p><img src="shift/shiftex.png">         
<p>
For each interval i (i=1, . . . , I) there is a function g<sub>i</sub> representing
the service level as a function of the
number of employees working in that interval.
Let y<sub>k</sub> be the number of employees of shift k for k = 1, . . . ,K.
Then the number of employees h<sub>i</sub>(y) working at interval i is given by
<p><img src="shift/def-hiy.png">
<p>
     The value of g<sub>i</sub> at this point is the attained service level in
interval i. The overall service level is defined by
<p><img src="shift/Sygi.png">
     <p>
Now consider the following problem, which maximizes
the service level for a given number of employees:
<p>     <img src="shift/maxSy.png">     
<p>     
  We assume that call arrivals are Poisson.
The service criterion is the fraction of customers that has to wait
longer than c seconds, e.g., c=11 seconds, before getting an operator,
which should be below 5% in general.
(We assume that no customers leave the system before getting
an operator.)
<p>
     Assuming that a statistical equilibrium is attained in
each time interval, we use the formula for the stationary waiting
time in an M|M|n queue as the service level.
Let
     <img src="shift/def-Lambda.png" align="middle">
<p>
     then the expected service level
of an arbitrary customer under schedule y is equal to
<p>
          <img src="shift/SyPW.png">
<p>     
where      <img src="shift/nihi.png"> 
 and
     <img src="shift/def-W.png"> 
are the waiting time
in an M|M|n queue with arrival rate  <img src="lambda.png"> 
     and service rate
 <img src="shift/mu.png"> .
     Thus we take
<p>      <img src="shift/def-gi.png"> <p>
This function is indeed monotone increasing and concave for each i.
     Therefore,  <img src="shift/minusS.png"> is multimodular. 
We can minimize <img src="shift/minusS.png"> by ODICON because
     multimodular functions and L$NATURAL-convex functions
     are equivalent objects
that can be related through a simple coordinate transformation.

EOF
#    print p, 'n: number of items', ln;
    print start_form(-name=>'p2',
		     -action=>'javascript:popup_window(document.p2)'),
    hidden('dim'), ln;
#
#    @index = (1..15);
#    print p, 'n = ', popup_menu(-name=>'newdim',-values=>[@index],-default=>$dim);
#    print submit('Change');

    print p, image('shift/c'), ': Service criterion (We measure the fraction of
customers that has to wait longer than ' , image('shift/c'), ' seconds before
getting an operator)', br, ln;

    print p, image('shift/c'), ' = ', hidden('criterion:min'), hidden('criterion:max'),
             textfield(-name=>'criterion', -size=>4,
                       -default=>param('criterion'), @MOUSE) , '(seconds)',ln;

    print p, image('shift/mu'),    ': Service rate per hour',br, ln;
    print p, image('shift/mu'), ' = ', hidden('mu:min'), hidden('mu:max'),
             textfield(-name=>'mu', -size=>4,
                       -default=>param('mu'), @MOUSE), br, ln;

    print image('shift/rate'),    ': Arrival rates per hour, ', 
    '0 &lt; ', image('shift/rate'), ' (', image('shift/i'), " = 1, 2, ... ,$dim)",br, ln;

    print start_table({-border=>'1'}), ln;
    print Tr({-align=>'right'}, [td({-width=>int(100/(1+$dim)).'%'}, [ image('shift/i'), 1 .. $dim ])]);
    foreach my $e ('rate') {
	my @td = (image("shift/$e")).hidden("$e:min").hidden("$e:max");
	for (my $j=0; $j<$dim; $j++) {
	    push @td, textfield(-name=>"$e:$j", -size=>3,
                                -default=>param("$e:$j"), @MOUSE).ln;
	}
	print Tr({-align=>'right'}, [td([@td])]), ln;
    }

#    foreach $e ('y') {
#	my @td = ($e);
#	for (my $j=0; $j<4; $j++) {
#	    push @td, textfield(-name=>"$e:$j", -size=>3, -default=>param("$e:$j"));
#	}
#	print Tr({-align=>'right'}, [td([@td])]), ln;
#    }
    {
        print hidden("y:min"), hidden("y:max"), ln;
	print Tr({-align=>'right'},
		 ['<td>y<sub>1</sub></td><td align="center" colspan=5 bgcolor="#EEFFCC">'.
		  textfield(-name=>"y:0", -size=>4, -default=>param("y:0"), @MOUSE).'</td><td colspan=8>&nbsp;</td>']), ln;
	print Tr({-align=>'right'},
		 ['<td>y<sub>2</sub></td><td colspan=2></td><td align="center" colspan=5 bgcolor="#EEFFCC">'.
		  textfield(-name=>"y:1", -size=>4, -default=>param("y:1"), @MOUSE).'</td><td colspan=6>&nbsp;</td>']), ln;
	print Tr({-align=>'right'},
		 ['<td>y<sub>3</sub></td><td colspan=5></td><td align="center" colspan=5 bgcolor="#EEFFCC">'.
		  textfield(-name=>"y:2", -size=>4, -default=>param("y:2"), @MOUSE).'</td><td colspan=3>&nbsp;</td>']), ln;
	print Tr({-align=>'right'},
		 ['<td>y<sub>4</sub></td><td colspan=8></td><td align="center" colspan=5 bgcolor="#EEFFCC">'.
		  textfield(-name=>"y:3", -size=>4, -default=>param("y:3"), @MOUSE).'</td>']), ln;
    }

  
    print end_table, ln;

    print p, 'You can input parameters ',  image('shift/c'),
    ' in the ranges 0 &lt; c &lt; 20',
    ' and ',  image('shift/mu'), ' , ',  image('shift/rate'),
    ' in the ranges 0 &lt; ',  image('shift/mu'), ' , ', image('shift/rate'),
    ' &lt; 100. ', ln;
    #image('shift/mu'), ' and ', image('shift/rate'),
    #' should satisfy the condition ', 
    #image('shift/rate'), ' &lt; ', image('shift/Nmu'), '.',
#    ', the unsatisfactory condition turns red.',
  ln;
    print p, 'You can also input an initial solution y.', ln,
    'The input should satisfy ', image('shift/hiy'), image('shift/mu'),
    ' &gt; ', image('shift/rate'), ' for all i. ',
    'Then, ', ln;
    print p, "N = y<sub>1</sub>+y<sub>2</sub>+y<sub>3</sub>+y<sub>4</sub> = ",
    textfield(-name=>"n_total", -size=>4, @MOUSE, -disabled), ' .', ln;

    print p, '(Note: "Reset" button resets your inputs to default values.)', ln;
    print p, submit(-name=>'Optimize', -disabled), ln,
    reset(-name=>'Reset', -onClick=>"return init()"), ln,
    end_form, ln;

    print br, ln;


#    print p, "We minimize the following cost function f(x):\n";
#    print image('objfunc'), ',', br, ln;
#    print 'where ', image('x'),       ': Spare amount of item ', image('j'),
#    ', 0 &le; ', image('x'), '(', image('j'), ' = 1, 2, ... ,n)',br, ln;
    print <<EOF;

<p>
<!--
The first term is the backorder penalty which is computed as
the cost per backorder times
the steady-state expected number of maximum backorders among the items.
The second term is the cost of spares purchases.

<p>
This function is L$NATURAL-convex [*]. -->
This web application minimizes
<!-- f(x), which transformed from -->
  <img src="shift/1minusSy.png"> using
<a href="http://ist.ksc.kwansei.ac.jp/~tutimura/odicon/">ODICON</a>
<!--with IFF by Satoru Iwata-->.
EOF
}

sub print_reference {
    print <<EOF;
<hr width="70%">

<p> [1] G. Koole and E. van der Sluis (2003):
  Optimal shift scheduling with a global service level
constraint,
IIE Transactions,
  Vol. 35, pp. 1049-1055.

<hr width="70%">
EOF
}

init_param();
print_header();
print_html_table();
print_reference();

print
    '<div align="right"><address>Satoko Moriguchi, Nobuyuki Tsuchimura</address></div>',ln,
    end_blockquote, end_html, ln;
